# Drawable objects are simply objects that are used for drawing to the screen.
# They have two purposes
# 1) To decouple rendering of objects from their creation
# 2) To allow for convenient reproduction of a figure and allow this
# representation to be serialised and restored.

class undef:
    pass

class Drawable(object):
    def fill(self, color=None):
        self.style = self.style._replace(fill=color)
        return self
    
    def outline(self, color=undef, weight=undef, join=undef, cap=undef):
        d = {}

        if weight is not undef: d['weight'] = weight
        if join is not undef: d['join'] = join
        if cap is not undef: d['cap'] = cap
        if color is not undef: d['outline'] = color

        self.style = self.style._replace(**d)
        return self


class Rectangle(Drawable):
    def __init__(self, a, b, style):
        self.a = a
        self.b = b
        self.style = style

    def extents(self):
        return self.a + self.b

class Gradient(Drawable):
    def __init__(self, a, b, stops):
        self.a = a
        self.b = b
        self.stops = stops

    def extents(self):
        return tuple(self.a) + tuple(self.b)

class Path(Drawable):
    def __init__(self, points, style, arrows=None):
        super(Path, self).__init__()
        self.style = style
        self.points = points
        self.arrows = arrows or []

    def __repr__(self):
        return "<Path(%s, %s)>" % (self.points, self.style)

    def extents(self):
        xs, ys = zip(*self.points)
        return min(xs), min(ys), max(xs), max(ys)

def combine_extents(extents):
    x0s, y0s, x1s, y1s = zip(*extents)
    return min(x0s), min(y0s), max(x1s), max(y1s)

class Picture(list):
    margin = 10, 10
    def extents(self):
        try:
            x0, y0, x1, y1 = combine_extents([i.extents() for i in self])
        except ValueError:
            x0, y0, x1, y1 = 0, 0, 0, 0
        mx, my = self.margin
        return x0 - mx, y0 - my, x1 + mx, y1 + my
    
    def append(self, arg):
        result = list.append(self, arg)
        self.on_update(self)
        return result

    def pop(self, ix=-1):
        result = list.pop(self, ix)
        self.on_update(self)
        return result

    @staticmethod
    def on_update(self):
        pass

class OpenPath(Path):
    pass

class ClosedPath(Path):
    pass

TP2PT = 72.00 / 72.27
from math import cos, sin

# There is a bug here.
class Text(Drawable):
    def __init__(self, tex, pos, style, anchor, angle):
        self.text = tex
        w, h = tex.size
        self.anchor = anchor
        self._width = w * TP2PT
        self._height = h * TP2PT
        self.pos = pos
        self.fill_color = style.fill
        self.angle = angle
        rx0, ry0, rx1, ry1 = self.extents()

        self.width = rx1 - rx0
        self.height = ry1 - ry0

    def fill(self, fill):
        self.fill_color = fill

    def extents(self):
        s = sin(self.angle)
        c = cos(self.angle)
        x, y = self.pos
        ax, ay = self.anchor
        w = self._width
        h = self._height
        # What is the unrotated bottom left?
        blx, bly = self.text.bl

        if ay is None:
            ay = bly

        x0 = - ax * w
        y0 = -ay * h
        x1 = (1-ax) * w
        y1 = (1-ay) * h

        rx0, ry0 = x + x0 * c - y0 * s, y + s * x0 + c * y0
        rx1, ry1 = x + x1 * c - y1 * s, y + s * x1 + c * y1

        return rx0, ry0, rx1, ry1

class Ellipse(Drawable):
    def __init__(self, a, b, root=(0, 0)):
        self.text = tex
        self.root = root
        self.axes = a, b


class Image(object):
    def __init__(self, data, p1, p2, width, height, filtering='nn'):
        self.data = data
        self.filtering = filtering
        self.width = width
        self.height = height
        self.p1 = p1
        self.p2 = p2

    def extents(self):
        x1, y1 = self.p1
        x2, y2 = self.p2
        return x1, y1, x2, y2

class Clip(object):
    def __init__(self, contents, region):
        self.contents = contents
        self.region = region

    def extents(self):
        return self.region.extents()

    def __repr__(self):
        return "Clip(contents=[...], region=%r)" % (self.region)


class Group(object):
    def __init__(self, contents):
        self._contents = contents

    def __repr__(self):
        return "%s%s" % (self.__class__.__name__, self.contents)




from ctypes import (c_void_p, c_char_p, c_uint32, CDLL, byref, POINTER,
                    Structure, c_byte, c_double)
import sys
from functools import partial
import struct

# PyCairo doesn't have any freetype support due to there being no python
# extension for freetype that exposes a C api. Since there is fat chance of this
# happening, as it's a lot of work, it's perhaps easiest to just wrap the
# functions we need. There's only a few needed for interop.

class PycairoContext(Structure):
    _fields_ = [("PyObject_HEAD", c_byte *
                 object.__basicsize__),
                ("ctx", c_void_p),
                ("base", c_void_p)]


# Freetype2 encoding enumerations are defined by packing a 32-bit bigendian
# integer with four bytes. Kinda like how the struct module does in python.
ADOBE_CUSTOM, = struct.unpack('>I', 'ADBC')
UNICODE, = struct.unpack('>I', 'unic')

ftlib = CDLL('libfreetype.so')

class FTFace(object):
    def __init__(self, ft_face):
        self._ft_face = ft_face
        self._as_parameter_ = ft_face
        self.select_charmap = partial(FTFace.select_charmap, self)
        self.get_char_index = partial(FTFace.get_char_index, self)
        self.get_name_index = partial(FTFace.get_name_index, self)

    _lib = ftlib
    get_name_index = _lib.FT_Get_Name_Index
    get_name_index.argtypes = (c_void_p, c_char_p)

    select_charmap = _lib.FT_Select_Charmap
    select_charmap.argypes = (c_void_p, c_uint32)

    get_char_index = _lib.FT_Get_Char_Index
    get_char_index.argtypes = (c_void_p, c_uint32)

class Freetype:
    # TODO: Will want to do some OSX detection here

    cache = {}

    def __init__(self):
        self._ft_lib = c_void_p()
        if self.init_freetype(byref(self._ft_lib)):
            raise RuntimeError("Freetype initialisation failed")

    def new_face(self, filename):
        """Implements a very simple cache"""
        ft_face = self.cache.get(filename)
        if not ft_face:
            ft_face = c_void_p()
            if Freetype._new_face(self._ft_lib, filename, 0, byref(ft_face)):
                raise RuntimeError(
                    "Error creating FreeType font face for " + filename)

            ft_face = FTFace(ft_face)
            ft_face.select_charmap(ADOBE_CUSTOM)
            self.cache[filename] = ft_face

        return ft_face


    _lib = ftlib

    init_freetype = _lib.FT_Init_FreeType
    init_freetype.argtypes = c_void_p,

    _new_face = _lib.FT_New_Face
    _new_face.argtypes = c_void_p, c_char_p, c_uint32, c_void_p

class Cairo:
    #TODO: OSX detection here
    _lib = CDLL('libcairo.so')

    create_from_ft_face = _lib.cairo_ft_font_face_create_for_ft_face
    create_from_ft_face.argtypes = c_void_p, c_uint32

    set_font_face = _lib.cairo_set_font_face
    set_font_face.argtypes = c_void_p, c_void_p

    font_face_status = _lib.cairo_font_face_status
    font_face_status.argtypes = c_void_p,

    font_options_create = _lib.cairo_font_options_create
    font_options_set_antialias = _lib.cairo_font_options_set_antialias
    font_options_set_subpixel_order\
            = _lib.cairo_font_options_set_subpixel_order
    font_options_set_hint_style = _lib.cairo_font_options_set_hint_style

    status = _lib.cairo_status
    status.argtypes = c_void_p,

    @classmethod
    def cr_face_from_ft_face(cls, ft_face):
        cr_face = cls.create_from_ft_face(ft_face, 0)
        if cls.font_face_status(cr_face):
            raise RuntimeError()
        return cr_face

    @classmethod
    def set_font_face(cls, cairo_ctx, cr_face):
        cairo_t = PycairoContext.from_address(id(cairo_ctx)).ctx

        cls._lib.cairo_set_font_face (cairo_t, cr_face)
        if cls.status (cairo_t):
            raise RuntimeError("Setting font face failed")

        face = cairo_ctx.get_font_face ()

freetype = Freetype()



# External modules
import cairo
from functools import partial

# Internal modules
from dvidraw.drawable import *
from dvidraw.freetype import Cairo, freetype, ADOBE_CUSTOM, UNICODE

# Dvipy modules
from dvipy.render import Char, Rule
from dvipy.font import kpse

from math import radians

TP2PT = 7200.0 / 7227.0

caps = {
    'round':cairo.LINE_CAP_ROUND,
    'butt':cairo.LINE_CAP_BUTT
}

joins = {
    'round':cairo.LINE_JOIN_ROUND
}

def path(path, cr, transform, closed=False):
    points = path.points
    x, y = transform(points[0])
    cr.move_to(x, y)
    for p in points[1:]:
        cr.line_to(*transform(p))
    if closed:
        cr.close_path()
    # Path space
    # Path space

def clip(clip, target, transform):
    target.save()
    region = clip.region
    contents = clip.contents
    # and now the horrible hack
    dispatcher[region.__class__](region, target, transform)
    target.clip()
    target.new_path()
    render(contents, target, transform)
    target.restore()

def rect(rect, target, transform):
    x1, y1 = transform(rect.a)
    x2, y2 = transform(rect.b)
    dx, dy = x2 - x1, y2 - y1
    target.rectangle(x1, y1, dx, dy)

def group(members, target, transform):
    for member in members:
        render(member, target, transform)

def text(member, target, transform):
    print member.anchor
    # Text that isn't filled shouldn't be drawn. Should probably print a
    # warning when this happens.
    if not member.fill_color:
        return

    # Push current transformation to the stack
    target.save()
    target.translate(*transform(member.pos))
    target.scale(TP2PT, TP2PT)
    target.rotate(radians(member.angle))

    # Get the bottom left of the text
    blx, bly = member.text.bl
    ax, ay = member.anchor
    sx, sy = member.text.size

    if ay is None:
        target.translate(-blx - sx * ax, 0)
    else:
        target.translate(-blx - sx * ax, -bly + sy * ay)

    texes = member.text.page
    

    glyphs = [t for t in texes if t.__class__ is Char]
    rules = [t for t in texes if t.__class__ is Rule]

    getter = operator.itemgetter(3)

    glyphs.sort(key=getter)
    for font, texlets in groupby(glyphs, key=getter):
        filename = font.filename
        face = (ft_cache.get(filename) or
                freetype.new_face(kpse(filename)))
        ft_cache[filename] = face

        cairo_face = (cairo_cache.get(face) or
                      Cairo.cr_face_from_ft_face(face))
        cairo_cache[face] = cairo_face

        Cairo.set_font_face(target, cairo_face)
        target.set_font_size(font.size)

        face.select_charmap(ADOBE_CUSTOM)

        glyphs = []
        for x, y, char, f in texlets:
            if isinstance(char, basestring):
                c = face.get_name_index(char)
            else:
                c = face.get_char_index(char)
            glyphs.append((c, x, y))

        target.set_source_rgb(*member.fill_color.rgb)
        face.select_charmap(UNICODE)
        target.show_glyphs(glyphs)

    target.restore()

def picture(picture, target, transform):
    cr = target

    for member in picture:
        f = dispatcher[member.__class__]
        f(member, target, transform)

        if hasattr(member, 'style'):
            style = member.style

            if style.weight:
                cr.set_line_width(style.weight)
            if style.fill and style.outline and style.weight:
                cr.set_source_rgb(*style.fill.rgb)
                cr.fill_preserve()
            if style.outline and style.weight:
                cr.set_source_rgb(*style.outline.rgb)
                cr.set_line_cap(caps[style.cap])
                cr.set_line_join(caps[style.join])
                cr.stroke()
            elif style.fill:
                cr.set_source_rgb(*style.fill.rgb)
                cr.fill()


def image(im, target, transform):
    imsurf = cairo.ImageSurface.create_for_data(
        im.data, cairo.FORMAT_ARGB32,
        im.width, im.height)

    im.pattern = cairo.SurfacePattern(imsurf) 
    if im.filtering == 'nn':
        im.pattern.set_filter(cairo.FILTER_NEAREST)
    else:
        raise AttributeError(
            'Unknown filtering method proposed, %s' %
        im.filtering)

    x1, y1 = transform(im.p1)
    x2, y2 = transform(im.p2)
    dx, dy = abs(x2 - x1), abs(y2 - y1)

    target.save()
    target.translate(x1, y2)
    print dx, dy
    target.scale(dx / im.width, dy / im.height)
    target.set_source(im.pattern)

    target.paint()
    target.restore()
    

def gradient(gradient, target, transform):
    x1, y1 = transform(gradient.p1)
    x2, y2 = transform(gradient.p2)

    g = cairo.LinearGradient(1, abs(y2-y1))
    for x, c in gradient.stops:
        g.add_color_stop_rgb(x, *c)

    target.save()
    target.translate(x1, x2)

    target.paint()
    target.restore()


# Later we can make the picture actually something sensible. is it a container?
render = picture

# The cursor should hold a renderer. The default can be overwritten. Each cursor
# has a render attribute. cr.save.svg()

dispatcher = {
    Clip : clip,
    Group : group,
    Rectangle: rect,
    OpenPath : partial(path, closed=False),
    ClosedPath : partial(path, closed=True),
    Text : text,
    Picture : picture,
    Image : image,
    Gradient: Gradient,
}
# ==============
# Text rendering
# ==============

import os.path
from math import ceil

class Saver(object):
    def __init__(self, picture):
        self.picture = picture

    def png(self, filename, dpi=90):
        dpp = dpi / 72.0
        x0, y0, x1, y1 = self.picture.extents()
        w = x1 - x0
        h = y1 - y0

        transform = lambda (x, y): (x - x0, h - (y - y0))
        surf = cairo.ImageSurface(cairo.FORMAT_RGB24, int(ceil(w*dpp)),
                                  int(ceil(h*dpp)))
        cr = cairo.Context(surf)
        cr.scale(dpp, dpp)
        cr.set_source_rgb(1, 1, 1)
        cr.paint()
        picture(self.picture, cr, transform)
        surf.write_to_png(filename)


    def pdf(self, filename):
        x0, y0, x1, y1 = self.picture.extents()
        w = x1 - x0
        h = y1 - y0

        transform = lambda (x, y): (x - x0, h - (y - y0))
        surf = cairo.PDFSurface(filename, w, h)
        cr = cairo.Context(surf)
        picture(self.picture, cr, transform)
        surf.finish()

    def screen(self):
        from dvidraw.cr_window import CairoDisplay
        x0, y0, x1, y1 = self.picture.extents()
        screen = CairoDisplay(self.picture, render)
        screen.show()

    dispatch = {'png' : png,
                'pdf' : pdf}

    def __call__(self, fname, *args, **kwargs):
        base, ext = os.path.splitext(fname)
        ext = ext[1:]
        f = self.dispatch[ext](self, fname, *args, **kwargs)
    

ft_cache = {}
cairo_cache = {}

import operator
from itertools import groupby




#External modules
from math import hypot
from functools import partial
import contextlib
import collections

# Interstitial modules
import dvipy.runner

# Internal drawing modules
from dvidraw import color
from dvidraw.drawable import (OpenPath, ClosedPath,
                           Text, Clip, Rectangle, Picture, Image)
import dvidraw.cr_draw

# Internal spaces modules
from dvidraw.bijections import bilog, bilin
from dvidraw.units import (UnitChanger, Coord, RawCoord, Point, DEFAULT_UNITS,
                        DEFAULT_UNIT)


# Create the session runner. We rely on python to clean this up.
runner = dvipy.runner.TexRunner()

# Override the default types on bilog and bilin for the common case
bilog = partial(bilog, a=RawCoord, b=RawCoord)
bilin = partial(bilin, a=RawCoord, b=RawCoord)

Style = collections.namedtuple('Style', ('fill', 'outline', 'weight', 'cap',
                                         'join'))
def norm(x, y):
    """Normalise a 2d point"""
    h = 1.0 / hypot(x, y)
    return (x * h, y * h)


# A descriptor that just happens to return a callable that moves the cursor
# given distance.
class CursorMover(object):
    """Move a cursor"""
    __slots__ = ('pos', 'target','__doc__','name')

    def __init__(self, dx, dy, name):
        self.pos = norm(dx, dy)
        self.name = name

    def __set__(self, *args):
        raise AttributeError('Cannot set')

    @property
    def __doc__(self):
        x, y = self.pos
        return ("""Returns a callable f(d) which moves in the direction of
                (%.2g, %.2g) by d units upon calling"""
                % (x, y))

    def __get__(self, ins, cls):
        self.target = ins
        return self

    def __call__(self, d):
        dx, dy = self.pos
        self.target.move(dx, dy, d)
        return self.target

    def __repr__(self):
        x, y = self.pos
        return "CursorMover(%f, %f, %s)" % (x, y, self.name)

# For now, it will be *absolutely* easiest to just transform everything with
# great regularity. For instance, a zoom simply pops an intermediate layer
# atop every space. 

class SpaceChanger(object):
    """Change space"""
    __slots__ = 'unit', '__doc__'

    @property
    def __doc__(self):
        return "Change unit to %s" % self.unit.name

    def __init__(self, unit):
        self.unit = unit

    def __get__(self, ins, cls):
        if ins:
            return ins.switch_space(self.unit)
        else:
            return self

    def __set__(self, *args):
        raise AttributeError('Unable to set UnitChanger')
    def __repr__(self):
        return "Change units to %s" % self.unit.name

# There's a difference between *easy* and *good*. Easy would be to hack in the
# device transformation here (that is flip upside down).
# But the right thing to do is clearly to put this into the cairo renderer,
# which can pick up the size. :). In this way, madness is spared.

UNIT_BOXSPACE = bilin((0, 0), (1, 1), a=Coord, b=Coord)
class Cursor(object):
    box = SpaceChanger('box')
    abs = SpaceChanger('abs')
    rel = SpaceChanger('rel')
    plt = SpaceChanger('plt')
    runner = runner

    def __init__(self, x=0, y=0, unit='pt'):
        self.coord = Coord(x, y)
        self.unit = DEFAULT_UNIT
        self.stack = []

        self.spaces = {'abs' : UNIT_BOXSPACE }
        self.current = self.spaces['abs']
        self.space_stack = []

        self.picture = Picture()
        black = self.rgb(0, 0, 0)
        self.style = Style(None, black, 1.0, 'round', 'round')
        self.save = dvidraw.cr_draw.Saver(self.picture)

    # SPACE COMMANDS
    def switch_unit(self, unit=None):
        """Change the unit of the underlying coordate"""
        if unit:
            self.unit = unit

        if self.unit == self.coord.unit:
            return self

        try:
            self.coord = self.coord.switch_unit(self.unit)
            return self
        except AttributeError:
            # Do the right thing: switch to relative space or fail and
            # get the root space
            new_space = self.spaces.get('rel') or self.spaces.get('abs')
            return self._switch_space(new_space).switch_unit(unit)

    def switch_space(self, name):
        return self._switch_space(self.spaces[name])
    
    def move(self, dx=0, dy=0, d=1):
        """Move by the given amount"""
        x, y = self.coord.point
        return self(x + dx*d, y + dy*d)

    def _switch_space(self, new_space):
        self.coord = new_space.a(*new_space.ba(self.anchored))

        self.current = new_space

        if self.coord.units:
            self.switch_unit()

        return self


    def zoom(self):
        # Much of this problem comes from the fear of "changing things"
        a = self.stack.pop()
        b = self.anchored

        # But really all that we're doing is changing the relative and box
        # mappings. Plots will be largely unaffected.
        d = b - a
        
        # How do we transform from the current space into the zoomed region?
        box = bilin(a, b, b=Coord)
        rel = bilin((0, 0), d, b=Coord).inverse + box
        
        self.space_stack.append(self.spaces.copy())
        self.spaces.update(box = box, rel = rel)
        return self

    def unzoom(self):
        self.spaces = self.space_stack.pop()
        return self

    def __call__(self, x=None, y=None):
        self.coord = self.coord(x, y)
        return self

    @property
    def anchored(self):
        return Point._make(self.current(self.coord.base))

    @property
    def to(self):
        """Push current point to the stack"""
        self.stack.append(self.anchored)
        return self

    def __repr__(self):
        x, y = self.coord.point
        if self.coord.unit:
            return 'Cursor((%g, %g) %s)' % (x, y, self.coord.unit.name)
        else:
            return 'Cursor(%g, %g)' % (x, y)

    # Drawing commands
    def clear(self):
        self.picture.clear()
        self.stack = []

    def image(self, data, width, height, filtering='nn'):
        self.to
        p2 = self.stack.pop()
        p1 = self.stack.pop()
        self.picture.append(Image(data, p1, p2, width, height, filtering))

    def gradient(self, stops):
        self.to
        p2 = self.stack.pop()
        p1 = self.stack.pop()
        self.picture.append(Gradient(p1, p2, stops))


    def path(self, close=False):
        """
        Turn the stack into a path

        If close is true, close the path by joining the begging and the end.

        """
        self.to
        stack = self.stack

        self.stack = []

        if close:
            p = ClosedPath(stack, self.style)
        else:
            p = OpenPath(stack, self.style)
        self.picture.append(p)
        return p

    def rect(self):
        """
        Turn the stack into a path

        If close is true, close the path by joining the begging and the end.
=
        """
        stack = self.stack
        self.to
        b = stack.pop()
        a = stack.pop()
        r = Rectangle(a, b, self.style)
        self.picture.append(r)
        return r

    def draw_text(self, text):
        text.pos = self.anchored
        self.picture.append(text)

    def text(self, tex, anchor=(0, None), draw=True, angle=0,
             fill_color=color.RGB(0, 0, 0)):
        page = runner.page(tex)
        text = Text(page, self.anchored, self.style, anchor, angle)
        text.fill_color = fill_color
        if draw:
            self.picture.append(text)
        return text

    def undo(self):
        try:
            self.picture.pop()
        except IndexError:
            raise RuntimeError('No more undo levels')

    # Clipping is achieved by obtaining a context. When the context is released,
    # clipping is over. This means it's impossible to "forget" to release the
    # clip, or at least very difficult.

    @property
    @contextlib.contextmanager
    def clip(self):
        region = self.picture.pop()

        picture, self.picture = self.picture, Picture()
        yield self
        picture.append(Clip(self.picture, region))
        self.picture = picture

    # STYLES

    def default(self, **kwargs):
        self.style = self.style._replace(**kwargs)
        
        return self

    def rgb(self, r, g, b):
        return color.RGB(r, g, b)

    def hsl(self, h, s, l):
        return color.HSL(h, s, l)


for i, j in DEFAULT_UNITS.iteritems():
    setattr(Cursor, i, UnitChanger(j))

MOVERS = (
    (CursorMover(1, 0, 'right'), ('right', 'E')),
    (CursorMover(0, 1, 'up'), ('up', 'N')),
    (CursorMover(-1, 0, 'left'), ('left', 'W')),
    (CursorMover(0, -1, 'down'), ('down', 'S')),
    (CursorMover(-1, -1, 'South-West'), ('SW',)),
    (CursorMover(+1, -1, 'South-East'), ('SE',)),
    (CursorMover(-1, +1, 'North-West'), ('NW',)),
    (CursorMover(+1, +1, 'North-East'), ('NE',))
)


for mover, bindings in MOVERS:
    for b in bindings:
        setattr(Cursor, b, mover)



import gtk
import time
import __main__ as main

def refresh_gui(delay=0):
    while gtk.events_pending():
        gtk.main_iteration_do(block=False)
        time.sleep(delay)

class CairoDisplay(object):
    running = False
    def __init__(self, picture, render):
        self.picture = picture
        self.render = render

    def init_window(self):
        # The gtk and cairo modules are only imported here
        self.window = gtk.Window()
        self.da = gtk.DrawingArea()
        self.window.add(self.da)
        self.window.show_all()
        self.window.connect('delete-event', self.on_delete)
        self.da.connect('expose-event', self.on_expose)

    def on_delete(self, win, evt):
        gtk.main_quit()
        self.running = False
        del self.picture.on_update

    def on_expose(self, win, evt):
        cr = win.window.cairo_create()
        cr.set_source_rgb(0, 0, 0)
        cr.paint()
        x0, y0, x1, y1 = self.picture.extents()
        w = x1 - x0
        h = y1 - y0

        win_w, win_h = win.window.get_size()
        scale = min(win_w / w, win_h / h)

        nw, nh = scale * w, scale * h

        dx = 0.5 * (win_w - nw)
        dy = 0.5 * (win_h - nh)

        cr.translate(dx, dy)
        cr.scale(scale, scale)

        cr.set_source_rgb(1, 1, 1)
        cr.rectangle(0, 0, w, h)
        cr.fill()

        transform = lambda (x, y): (x - x0, h - (y - y0))
        self.render(self.picture, cr, transform)

        pass

    def redraw(self, _):
        self.da.queue_draw()

    def show(self, delay=0, block=None):
        if self.running:
            return

        self.init_window()
        self.running = True

        self.picture.on_update = self.redraw

        if block is None:
            block = hasattr(main, '__file__')

        if not block:
            refresh_gui()
        else:
            gtk.main()



from distutils.core import setup

setup(name = 'dvidraw',
      packages = ['dvidraw'],
      version = '0.0.5',
      install_requires = ['dvipy'],
      author = 'Dave Mallows',
      author_email = 'dave.mallows@gmail.com',
      url = 'http://dpwm.bitbucket.org/dvidraw',
      description = 'Drawing package with LaTeX-rendered text',
      keywords = 'drawing dvi tex latex',
      license = 'GPLv2')


